@@ -289,7 +289,13 @@ sub task_run {
     }
 
     if ($spj->{plugin_type} eq 'outthentic'){
-      $cmd.="  strun --root ./ --task \"[task] ".($task_set->{task_desc})."\""
+
+      if ($^O  =~ 'MSWin') {
+        $cmd.="  strun --root ./ --task \"[task] ".($task_set->{task_desc})."\""
+      } else {
+        $cmd.="  strun --root ./ --task '[task] ".($task_set->{task_desc})."'"
+      }
+
     } elsif ( $spj->{plugin_type} eq 'swat' ) {
       #warn $task_set->{host};
       $cmd.="  swat ./ ". ($task_set->{host}).' ';
